package bytesrw

  1. Overview
  2. Docs

Module Bytesrw_zstd.Cctx_paramsSource

Compression parameters.

Sourcetype clevel = int

The type for compression levels. See clevel.

Sourcetype t

The type for compression parameters.

Sourceval make : ?init:t -> ?checksum:bool -> ?clevel:clevel -> ?window_log:int -> unit -> t

make () are the given compression parameters. Those unspecfied take the value of init which defaults to default. See corresponding accessors for the default values.

Sourceval default : t

default are the default parameters. See accessors for the default values.

Sourceval checksum : t -> bool

checksum p is true if frames are checksumed. Defaults to false Warning. This mirrors the library default but does not mirror the zstd tool default.

Sourceval clevel : t -> clevel

clevel p is the compression level. Must be in the min_clevel to max_clevel range. Defaults to default_clevel. 0 means default compression level.

Sourceval window_log : t -> int

window_log is the maximal allowed back-reference distance in power of 2. 0 means default window log.